package com.chesire.nekome.core.compose.theme

import android.os.Build
import android.util.Log
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.dynamicDarkColorScheme
import androidx.compose.material3.dynamicLightColorScheme
import androidx.compose.runtime.Composable
import androidx.compose.ui.platform.LocalContext
import com.google.accompanist.systemuicontroller.rememberSystemUiController

@Composable
fun NekomeTheme(
    isDarkTheme: Boolean = isSystemInDarkTheme(),
    isDynamicColor: Boolean = true,
    content: @Composable () -> Unit
) {
    val systemUiController = rememberSystemUiController()

    val dynamicColor = isDynamicColor && Build.VERSION.SDK_INT >= Build.VERSION_CODES.S
    val colorScheme = when {
        dynamicColor && isDarkTheme -> dynamicDarkColorScheme(LocalContext.current)
        dynamicColor && !isDarkTheme -> dynamicLightColorScheme(LocalContext.current)
        isDarkTheme -> DarkColorPalette
        else -> LightColorPalette
    }
    systemUiController.apply {
        setNavigationBarColor(color = colorScheme.surface)
        setStatusBarColor(color = colorScheme.background)
    }

    Log.d(
        "Nekome",
        "Is system in dark theme? [$isDarkTheme], is using dynamic color? [$isDynamicColor]"
    )

    MaterialTheme(
        colorScheme = colorScheme,
        typography = NekomeTypography,
        shapes = NekomeShapes,
        content = content
    )
}
