package com.chesire.nekome.core.flags

import android.content.Context
import androidx.annotation.StringRes
import com.chesire.nekome.resources.StringResource

/**
 * List of possible states a series can be in for a user.
 */
enum class UserSeriesStatus(val index: Int, @StringRes val stringId: Int) {
    Unknown(-1, 0),
    Current(0, StringResource.filter_by_current),
    Completed(1, StringResource.filter_by_completed),
    OnHold(2, StringResource.filter_by_on_hold),
    Dropped(3, StringResource.filter_by_dropped),
    Planned(4, StringResource.filter_by_planned);

    companion object {
        /**
         * Gets a map of [index] to the string acquired from the [stringId].
         * This call will ignore the [Unknown] value as it should not be displayed to the user.
         */
        fun getValueMap(context: Context) = values()
            .filterNot { it == Unknown }
            .associate { it.index to context.getString(it.stringId) }

        /**
         * Gets the [UserSeriesStatus] from a given [index], the [index] should be the index field
         * in the the enum class, but as a string. If this is null or cannot be found then [Unknown]
         * will be returned.
         */
        fun getFromIndex(index: String): UserSeriesStatus {
            return index.toIntOrNull()?.let { intIndex ->
                values().find { it.index == intIndex } ?: Unknown
            } ?: Unknown
        }
    }
}
